@@ -1,7 +1,6 @@
 //! *A*syncronous and *P*leasant *E*ntity *C*omponent *S*ystem
 //!
 #![feature(generic_associated_types)]
-#![feature(vec_retain_mut)]
 #![allow(clippy::type_complexity)]
 mod core;
 pub mod system;
@@ -16,3 +15,5 @@ pub mod anyhow {
 }
 
 pub use crate::core::*;
+
+pub use rustc_hash::FxHashMap;
