@@ -1,21 +1,25 @@
 //! The [`World`] contains resources and is responsible for ticking
 //! systems.
-use std::{future::Future, sync::Arc};
+use std::{future::Future, marker::PhantomData, sync::Arc};
 
 use anyhow::Context;
-use rustc_hash::FxHashMap;
+use rustc_hash::{FxHashMap, FxHashSet};
 
 use crate::{
+    entities::{Entities, Entity},
     mpsc,
-    plugins::IsPlugin,
+    plugins::{entity_upkeep::PluginEntityUpkeep, Plugin},
     schedule::{Borrow, IsBorrow, IsSchedule},
     spsc,
-    system::{AsyncSchedule, AsyncSystem, AsyncSystemRequest, SyncSchedule, SyncSystem, AsyncSystemFuture},
-    CanFetch, FetchReadyResource, IsResource, Request, Resource, ResourceId,
+    storage::{CanWriteStorage, StoredComponent, WorldStorage},
+    system::{
+        AsyncSchedule, AsyncSystem, AsyncSystemFuture, AsyncSystemRequest, ShouldContinue,
+        SyncSchedule, SyncSystem,
+    },
+    CanFetch, FetchReadyResource, IsResource, Request, Resource, ResourceId, ResourceRequirement,
+    Write,
 };
 
-pub use crate::plugins::entity_upkeep::EntityUpkeepExt;
-
 pub struct Facade {
     // Unbounded. Sending a request from the system should not yield the async
     pub(crate) resource_request_tx: spsc::Sender<Request>,
@@ -52,12 +56,34 @@ impl Facade {
     }
 }
 
+pub struct EntityBuilder<'a> {
+    entity: Entity,
+    world: &'a mut World,
+}
+
+impl<'a> EntityBuilder<'a> {
+    pub fn with<C: StoredComponent>(self, component: C) -> anyhow::Result<Self>
+    where
+        C::StorageType: WorldStorage,
+    {
+        if !self.world.has_resource::<C::StorageType>() {
+            self.world.with_default_storage::<C>()?;
+        }
+        let mut storage: Write<C::StorageType> = self.world.fetch()?;
+        let _ = storage.insert(self.entity.id(), component);
+        Ok(self)
+    }
+
+    pub fn build(self) -> Entity {
+        self.entity
+    }
+}
+
 pub struct World {
     // world resources
     resources: FxHashMap<ResourceId, Resource>,
-    // resources loaned out to the owner of
-    // this world
-    loaned_resources: FxHashMap<ResourceId, Arc<Resource>>,
+    // resources loaned out by reference
+    borrowed_resources: FxHashMap<ResourceId, Arc<Resource>>,
     // outer resource channel (unbounded)
     resources_from_system: (
         mpsc::Sender<(ResourceId, Resource)>,
@@ -74,7 +100,7 @@ impl Default for World {
     fn default() -> Self {
         Self {
             resources: Default::default(),
-            loaned_resources: FxHashMap::default(),
+            borrowed_resources: FxHashMap::default(),
             resources_from_system: mpsc::unbounded(),
             sync_schedule: SyncSchedule::default(),
             async_systems: vec![],
@@ -88,7 +114,7 @@ pub(crate) fn clear_returned_resources(
     system_name: Option<&str>,
     resources_from_system_rx: &mpsc::Receiver<(ResourceId, Resource)>,
     resources: &mut FxHashMap<ResourceId, Resource>,
-    loaned_resources: &mut FxHashMap<ResourceId, Arc<Resource>>,
+    borrowed_resources: &mut FxHashMap<ResourceId, Arc<Resource>>,
 ) -> anyhow::Result<()> {
     while let Ok((rez_id, resource)) = resources_from_system_rx.try_recv() {
         // put the resources back, there should be nothing stored there currently
@@ -102,8 +128,8 @@ pub(crate) fn clear_returned_resources(
     }
 
     // put the borrowed resources back
-    if !loaned_resources.is_empty() {
-        for (id, rez) in std::mem::take(loaned_resources).into_iter() {
+    if !borrowed_resources.is_empty() {
+        for (id, rez) in std::mem::take(borrowed_resources).into_iter() {
             let rez = Arc::try_unwrap(rez)
                 .map_err(|_| anyhow::anyhow!("could not retreive borrowed resource"))?;
             resources.insert(id, rez);
@@ -211,21 +237,39 @@ impl World {
         Ok(self)
     }
 
-    pub fn with_plugin<Plugin: IsPlugin>(&mut self, plugin: Plugin) -> &mut Self {
-        for lazy_rez in plugin.resources().into_iter() {
-            if !self.resources.contains_key(lazy_rez.id()) {
-                let (id, resource) = lazy_rez.into();
-                let _ = self.resources.insert(id, resource);
+    pub fn with_plugin(&mut self, plugin: impl Into<Plugin>) -> anyhow::Result<&mut Self> {
+        let plugin = plugin.into();
+
+        let mut missing_required_resources = FxHashSet::default();
+        for req_rez in plugin.resources.into_iter() {
+            if !self.resources.contains_key(req_rez.id()) {
+                match req_rez {
+                    ResourceRequirement::ExpectedExisting(id) => {
+                        missing_required_resources.insert(id);
+                    }
+                    ResourceRequirement::LazyDefault(lazy_rez) => {
+                        let (id, resource) = lazy_rez.into();
+                        missing_required_resources.remove(&id);
+                        let _ = self.resources.insert(id, resource);
+                    }
+                }
             }
         }
 
-        for system in plugin.sync_systems().into_iter() {
+        anyhow::ensure!(
+            missing_required_resources.is_empty(),
+            "missing required resources:\n{:#?}",
+            missing_required_resources
+        );
+
+        for system in plugin.sync_systems.into_iter() {
             if !self.sync_schedule.contains_system(&system.0.name) {
-                self.sync_schedule.add_system_with_dependecies(system.0, system.1.into_iter());
+                self.sync_schedule
+                    .add_system_with_dependecies(system.0, system.1.into_iter());
             }
         }
 
-        'outer: for asystem in plugin.async_systems().into_iter() {
+        'outer: for asystem in plugin.async_systems.into_iter() {
             for asys in self.async_systems.iter() {
                 if asys.name == asystem.0 {
                     continue 'outer;
@@ -235,26 +279,68 @@ impl World {
             let (sys, fut) = make_async_system_pack(&self, asystem.0, asystem.1);
             self.add_async_system_pack(sys, fut);
         }
-        self
+
+        Ok(self)
     }
 
-    pub fn with_system<T, F>(&mut self, name: impl AsRef<str>, sys_fn: F) -> &mut Self
+    pub fn with_data<T: CanFetch>(&mut self) -> anyhow::Result<&mut Self> {
+        self.with_plugin(T::plugin())
+    }
+
+    pub fn with_storage<T: StoredComponent>(
+        &mut self,
+        store: T::StorageType,
+    ) -> anyhow::Result<&mut Self> {
+        self.with_resource(store)?
+            .with_plugin(PluginEntityUpkeep::<T::StorageType>(PhantomData))
+    }
+
+    pub fn with_default_storage<T: StoredComponent>(&mut self) -> anyhow::Result<&mut Self> {
+        let store = <T::StorageType>::default();
+        self.with_resource(store)?
+            .with_plugin(PluginEntityUpkeep::<T::StorageType>(PhantomData))
+    }
+
+    pub fn entity(&mut self) -> anyhow::Result<EntityBuilder<'_>> {
+        if !self.has_resource::<Entities>() {
+            self.with_default_resource::<Entities>()?;
+        }
+        let mut entities = self.fetch::<Write<Entities>>()?;
+        let entity = entities.create();
+        Ok(EntityBuilder {
+            world: self,
+            entity,
+        })
+    }
+
+    pub fn with_system<T, F>(
+        &mut self,
+        name: impl AsRef<str>,
+        sys_fn: F,
+    ) -> anyhow::Result<&mut Self>
     where
-        F: FnMut(T) -> anyhow::Result<()> + Send + Sync + 'static,
+        F: FnMut(T) -> anyhow::Result<ShouldContinue> + Send + Sync + 'static,
         T: CanFetch,
     {
         self.with_system_with_dependencies(name, &[], sys_fn)
     }
 
-    pub fn with_system_with_dependencies<T, F>(&mut self, name: impl AsRef<str>, deps: &[&str], sys_fn: F) -> &mut Self
+    pub fn with_system_with_dependencies<T, F>(
+        &mut self,
+        name: impl AsRef<str>,
+        deps: &[&str],
+        sys_fn: F,
+    ) -> anyhow::Result<&mut Self>
     where
-        F: FnMut(T) -> anyhow::Result<()> + Send + Sync + 'static,
+        F: FnMut(T) -> anyhow::Result<ShouldContinue> + Send + Sync + 'static,
         T: CanFetch,
     {
         let system = SyncSystem::new(name, sys_fn);
 
-        self.sync_schedule.add_system_with_dependecies(system, deps.iter());
-        self
+        self.with_plugin(T::plugin())?;
+        self.sync_schedule
+            .add_system_with_dependecies(system, deps.iter());
+        Ok(self)
     }
 
     pub fn with_sync_systems_run_in_parallel(&mut self, parallelize: bool) -> &mut Self {
@@ -308,6 +394,12 @@ impl World {
         self
     }
 
+    /// Returns whether a resources of the given type exists in the world.
+    pub fn has_resource<T: IsResource>(&self) -> bool {
+        let rez_id = ResourceId::new::<T>();
+        self.resources.contains_key(&rez_id)
+    }
+
     /// Spawn a non-system asynchronous task.
     pub fn spawn(&self, future: impl Future<Output = ()> + Send + Sync + 'static) {
         let task = self.async_task_executor.spawn(future);
@@ -327,7 +419,7 @@ impl World {
             None,
             &self.resources_from_system.1,
             &mut self.resources,
-            &mut self.loaned_resources,
+            &mut self.borrowed_resources,
         )?;
 
         // run the scheduled sync systems
@@ -342,7 +434,7 @@ impl World {
             None,
             &self.resources_from_system.1,
             &mut self.resources,
-            &mut self.loaned_resources,
+            &mut self.borrowed_resources,
         )
         .unwrap();
 
@@ -392,12 +484,36 @@ impl World {
         Ok(())
     }
 
+    /// Attempt to get a reference to one resource.
+    pub fn resource_get<T: IsResource>(&self) -> anyhow::Result<&T> {
+        let id = ResourceId::new::<T>();
+        let box_t: &Resource = self
+            .resources
+            .get(&id)
+            .with_context(|| format!("resource {} is missing", id.name))?;
+        box_t
+            .downcast_ref()
+            .with_context(|| "could not downcast resource")
+    }
+
+    /// Attempt to get a mutable reference to one resource.
+    pub fn resource_get_mut<T: IsResource>(&mut self) -> anyhow::Result<&mut T> {
+        let id = ResourceId::new::<T>();
+        let box_t: &mut Resource = self
+            .resources
+            .get_mut(&id)
+            .with_context(|| format!("resource {} is missing", id.name))?;
+        box_t
+            .downcast_mut()
+            .with_context(|| "could not downcast resource")
+    }
+
     pub fn fetch<T: CanFetch>(&mut self) -> anyhow::Result<T> {
         clear_returned_resources(
             None,
             &self.resources_from_system.1,
             &mut self.resources,
-            &mut self.loaned_resources,
+            &mut self.borrowed_resources,
         )?;
 
         let reads = T::reads().into_iter().map(|id| Borrow {
@@ -411,7 +527,7 @@ impl World {
         let borrows = reads.chain(writes).collect::<Vec<_>>();
         let (mut rezs, staying_refs) =
             try_take_resources(&mut self.resources, borrows.iter(), None)?;
-        self.loaned_resources.extend(staying_refs);
+        self.borrowed_resources.extend(staying_refs);
         T::construct(self.resources_from_system.0.clone(), &mut rezs)
     }
 
@@ -438,7 +554,7 @@ impl World {
 #[cfg(test)]
 mod test {
     use crate as apecs;
-    use apecs::{anyhow, entities::*, join::*, spsc, storage::*, system, world::*, Read, Write};
+    use apecs::{anyhow, entities::*, join::*, spsc, storage::*, system::*, world::*, Read, Write};
 
     #[test]
     fn can_closure_system() -> anyhow::Result<()> {
@@ -449,19 +565,23 @@ mod test {
 
         fn mk_stateful_system(
             tx: spsc::Sender<(f32, f32)>,
-        ) -> impl FnMut(StatefulSystemData) -> anyhow::Result<()> {
+        ) -> impl FnMut(StatefulSystemData) -> anyhow::Result<ShouldContinue> {
+            println!("making stateful system");
             let mut highest_pos: (f32, f32) = (0.0, f32::NEG_INFINITY);
 
             move |data: StatefulSystemData| {
-                for (_, pos) in (&data.positions,).join() {
+                println!("running stateful system: highest_pos:{:?}", highest_pos);
+                for pos in data.positions.iter() {
                     if pos.1 > highest_pos.1 {
-                        highest_pos = *pos;
+                        highest_pos = *pos.value();
+                        println!("set new highest_pos: {:?}", highest_pos);
                     }
                 }
 
+                println!("sending highest_pos: {:?}", highest_pos);
                 tx.try_send(highest_pos)?;
 
-                Ok(())
+                ok()
             }
         }
 
@@ -475,7 +595,7 @@ mod test {
         let mut world = World::default();
         world
             .with_resource(positions)?
-            .with_system("stateful", mk_stateful_system(tx));
+            .with_system("stateful", mk_stateful_system(tx))?;
 
         world.tick()?;
 
@@ -510,14 +630,14 @@ mod test {
                 Read<VecStorage<u32>>,
                 Write<FxHashMap<String, u32>>,
             ),
-        ) -> anyhow::Result<()> {
-            for (_, name, number) in (&data.0, &data.1).join() {
-                if !data.2.contains_key(name) {
-                    let _ = data.2.insert(name.to_string(), *number);
+        ) -> anyhow::Result<ShouldContinue> {
+            for (name, number) in (&data.0, &data.1).join() {
+                if !data.2.contains_key(name.value()) {
+                    let _ = data.2.insert(name.to_string(), *number.value());
                 }
             }
 
-            Ok(())
+            ok()
         }
 
         let subscriber = tracing_subscriber::FmtSubscriber::builder()
@@ -530,11 +650,8 @@ mod test {
         let mut world = World::default();
         world
             .with_default_resource::<Entities>()?
-            .with_default_resource::<VecStorage<String>>()?
-            .with_default_resource::<VecStorage<u32>>()?
-            .with_default_resource::<FxHashMap<String, u32>>()?
-            .with_async_system("create", system::make_stateful_async_system(tx, create))
-            .with_system("maintain", maintain_map);
+            .with_async_system("create", |facade| async move { create(tx, facade).await })
+            .with_system("maintain", maintain_map)?;
 
         // create system runs - sending on the channel and making the fetch request
         world.tick()?;
@@ -551,4 +668,22 @@ mod test {
 
         Ok(())
     }
+
+    #[test]
+    fn can_create_entities_and_build_convenience() -> anyhow::Result<()> {
+        struct DataA(f32);
+        impl StoredComponent for DataA {
+            type StorageType = VecStorage<Self>;
+        }
+
+        struct DataB(f32);
+        impl StoredComponent for DataB {
+            type StorageType = VecStorage<Self>;
+        }
+
+        let mut world = World::default();
+        let _entity = world.entity()?.with(DataA(0.0))?.with(DataB(0.0))?.build();
+
+        Ok(())
+    }
 }
